@@ -5,7 +5,7 @@
  
 package XML::Compile::Translate::Writer;
 use vars '$VERSION';
-$VERSION = '1.19';
+$VERSION = '1.20';
 
 use base 'XML::Compile::Translate';
 
@@ -121,12 +121,11 @@ sub makeElementWrapper
 sub makeWrapperNs
 {   my ($self, $path, $processor, $index, $filter) = @_;
     my @entries;
-    $filter = sub {1} if ref $filter ne 'CODE';
+    $filter = sub {$_[2]} if ref $filter ne 'CODE'; # only the used
 
     foreach my $entry (sort {$a->{prefix} cmp $b->{prefix}} values %$index)
     {   # ANY components are frustrating this
-        $entry->{used} or next;
-        $filter->($entry->{uri}, $entry->{prefix}) or next;
+        $filter->($entry->{uri}, $entry->{prefix}, $entry->{used}) or next;
         push @entries, [ $entry->{uri}, $entry->{prefix} ];
         $entry->{used} = 0;
     }
